#!/usr/bin/python
# -*- coding: utf-8 -*-

from ... import ftdi

from . import constant
from .common import common

#########################################
CH_CNT = 1
CONFIG_LIST = [
    # common
    ["PDSW", constant.CHNUM_COM],
    # CH
    ["ADG", constant.CHNUM_CHN],
    ["THR", constant.CHNUM_CHN],
    ["LLD", constant.CHNUM_CHN],
    ["ULD", constant.CHNUM_CHN],
    ["OFS", constant.CHNUM_CHN],
]


class config(object):
    #########################################################
    def __init__(self, usbmca):
        self.mConfigList = CONFIG_LIST
        self.__initiale()
        self.mUSBMCA = usbmca

    #########################################################
    def __initiale(self):
        self.mUSBMCA = None
        self.mParameter = {}
        self.mLastException = None
        self.clearConfig()

    #########################################################
    def clearConfig(self):
        self.mParameter = {}
        zerovalue = [0 for n in range(CH_CNT)]

        for name, chn in self.mConfigList:
            if chn == constant.CHNUM_CHN:
                self.mParameter[name,] = zerovalue
            else:
                self.mParameter[name,] = 0

        return

    #########################################################
    def __getUseDestination(self, dest):
        _dest = dest if dest is not None else self.mUSBMCA
        return _dest

    #########################################################
    def setDefaultDest(self, usbmca):
        self.mUSBMCA = usbmca

    #########################################################
    def startAcquisition(self, dest=None):
        return self.writeParameter("AQSW", 1, constant.CHNUM_COM, dest)

    #########################################################
    def stopAcquisition(self, dest=None):
        return self.writeParameter("AQEW", 1, constant.CHNUM_COM, dest)

    #########################################################
    def clearAcquisitionData(self, dest=None):
        return self.writeParameter("CLRW", 1, constant.CHNUM_COM, dest)

    #########################################################
    def setAcquisitionMode(self, mode, dest=None):
        return self.writeParameter("MODW", mode, constant.CHNUM_COM, dest)

    #########################################################
    def setAcquisitionTimeMode(self, mode=0, dest=None):
        return self.writeParameter("MMDW", mode, constant.CHNUM_COM, dest)

    #########################################################
    def setAcquisitionTime(self, seconds, dest=None):
        result = False
        digit = common.convertTimeSecondToDigits(seconds)
        digit0 = (digit >> 32) & 0x0FFF
        digit1 = (digit >> 0) & 0x0FFFFFFFF

        result = self.writeParameter("MT0W", digit0, constant.CHNUM_COM, dest)  # MSB

        if result == True:
            result = self.writeParameter(
                "MT1W", digit1, constant.CHNUM_COM, dest
            )  # LSB

        return result

    #########################################################
    def writeParameter(self, command, value, chn, dest=None):
        _dest = self.__getUseDestination(dest)
        result = _dest.RequestCommand(command, value, chn)

        if result == True:
            try:
                dummy = self.mParameter[command,]
                if chn == constant.CHNUM_COM:
                    self.mParameter[command,] = value
                else:
                    self.mParameter[command,][chn - 1] = value
            except Exception as ex:
                self.mLastException = ex

        return result

    #########################################################
    def writeParameterAll(self, dest=None):
        result = True

        for (name,) in self.mParameter:
            if isinstance(self.mParameter[name,], list):
                value = self.mParameter[name,]  # CH unit
                chn = [constant.CHNUM_1ST + n for n in range(len(value))]
            else:
                value = [self.mParameter[name,]]  # common
                chn = [constant.CHNUM_COM]

            for i in range(len(value)):
                result = self.writeParameter(name, value[i], chn[i], dest)

                if result == False:
                    return result

        return result
