#!/usr/bin/python
# -*- coding: utf-8 -*-

import array

from ... import num
from ... import ftdi
from . import constant
from .common import common

READ_BYTES = 2048
READ_MAXBLK = 32


class Histogram(object):
    #####################################################################
    def __init__(self, dest):
        self.__initialize()
        self.mUSBMCA = dest
        self.mSpectrum = None

    #####################################################################
    def __initialize(self):
        self.mUSBMCA = None
        self.mLastException = None

    #########################################################
    def __getUseDestination(self, dest):
        _dest = dest if dest is not None else self.mUSBMCA
        return _dest

    #########################################################
    def read(self, blocks=READ_MAXBLK, dest=None):
        histdata = None
        usbmca = self.__getUseDestination(dest)
        result = 0

        try:
            rxdata = bytearray()

            for i in range(blocks):
                ret2nd, rxdata0 = self.__read(i, usbmca)

                if ret2nd != True:
                    result = -2
                    break

                rxdata += rxdata0
                result += 1

            histdata = num.bitconverter.copyToArrayU32(rxdata)

            if num.byteorder.isBigmachine() == False:
                histdata.byteswap()

            self.mSpectrum = histdata
        except Exception as ex:
            self.mLastException = ex
            result = -1

        return result, histdata

    #########################################################
    def __read(self, index, dest=None):
        result = False
        rxdata = None
        rxbyte = 0
        usbmca = self.__getUseDestination(dest)

        command = "HI%02X" % (index)
        ret0th, dummy = usbmca.RequestOnly(command, 0)

        if ret0th == ftdi.constant.FT_OK:
            ret1st, rxdata, rxbyte = usbmca.Read(READ_BYTES)

            if usbmca.isSuccess(ret1st) == True and rxbyte == READ_BYTES:
                result = True

        return result, rxdata
