#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import time
import copy
import array

from . import constant
from .common import common
from ... import num
from ... import ftdi

STATUS_LIST = [
    # common
    ["RLT", constant.CHNUM_COM, 6],
    # CH1
    ["LVT", constant.CHNUM_1ST, 6],
    ["DDT", constant.CHNUM_1ST, 6],
    ["TCR", constant.CHNUM_1ST, 3],
    ["TCT", constant.CHNUM_1ST, 4],
]

STATUS_BYTES = 25  # sum of above data


class status(object):
    #########################################################
    def __init__(self, usbmca=None):
        self.mStatusItemList = STATUS_LIST
        self.__initializing()
        self.mUSBMCA = usbmca

    #########################################################
    def __initializing(self):
        self.mUSBMCA = None
        self.clearStatus()

    #########################################################
    def clearStatus(self):
        self.mStatus = {}
        self.mSumBytes = 0

        for item0 in self.mStatusItemList:
            name = self.generateKey(item0[0], item0[1])
            self.mStatus[name,] = 0
            self.mSumBytes = self.mSumBytes + item0[2]

    #########################################################
    def readStatus(self, usbmca=None):
        result = False
        device0 = usbmca if usbmca is not None else self.mUSBMCA

        resulttx, txbytes = device0.RequestOnly("STUW", 0)

        if ftdi.constant.FT_SUCCESS(result) == True:
            resultrx, rxdata, rxbytes = device0.Read(STATUS_BYTES)

            if ftdi.constant.FT_SUCCESS(result) == True and rxbytes == STATUS_BYTES:
                index = 0
                temp0 = bytearray(8)

                for item0 in self.mStatusItemList:
                    #######
                    name = self.generateKey(item0[0], item0[1])

                    #######
                    value = copy.deepcopy(temp0)

                    if 0 < item0[2] and item0[2] < 8:
                        zlength = 8 - item0[2]
                        nlast = index + item0[2]
                        value = temp0[0:zlength] + rxdata[index:nlast]

                    value1 = num.bitconverter.toIntU64(value)
                    self.mStatus[name,] = num.byteorder.ntoh64(value1)
                    index = index + item0[2]

                result = True

        return result

    #########################################################
    def getRealTimeSec(self):
        time0 = common.convertTimeDigitToSeconds(self.mStatus["RLT",])
        return time0

    #########################################################
    def getLiveTimeSec(self, chn):
        name = status.generateKey("LVT", chn)
        value = common.convertTimeDigitToSeconds(self.mStatus[name,])
        return value

    #########################################################
    def getDeadTimeSec(self, chn):
        name = status.generateKey("DDT", chn)
        value = common.convertTimeDigitToSeconds(self.mStatus[name,])
        return value

    #########################################################
    @staticmethod
    def generateKey(name, chn):
        if chn == constant.CHNUM_COM:
            keyname = name
        else:
            keyname = "%s%d" % (name, chn)

        return keyname
