#!/usr/bin/python
# -*- coding: utf-8 -*-

##############################################################
import os
import sys
import ctypes

from ..device import device

LIBFTD2XX_32BIT = "libftd2xx.so.1.4.8"
LIBFTD2XX_64BIT = "libftd2xx.so.1.4.8"


##############################################################
def loadLibrary():
    libFTD2XX = None
    ostype = device.getOperatingSystem()

    if ostype == device.OS_TYPE_WINDOWS:
        libFTD2XX = ctypes.WinDLL("ftd2xx.dll")
    elif ostype == device.OS_TYPE_LINUX:
        __dir = os.path.dirname(__file__)

        if device.IS_64BIT == True:
            libpath = "%s/lib/linux64/%s" % (__dir, LIBFTD2XX_64BIT)
        else:
            libpath = "%s/lib/linux32/%s" % (__dir, LIBFTD2XX_32BIT)

        libFTD2XX = ctypes.cdll.LoadLibrary(libpath)

    return libFTD2XX


##############################################################
FTD2XXLIBHANDLE = loadLibrary()

##############################################################
# typedef PVOID   FT_HANDLE;
# typedef ULONG   FT_STATUS;

##############################################################
#
# Device status
#
FT_OK = 0
FT_INVALID_HANDLE = 1
FT_DEVICE_NOT_FOUND = 2
FT_DEVICE_NOT_OPENED = 3
FT_IO_ERROR = 4
FT_INSUFFICIENT_RESOURCES = 5
FT_INVALID_PARAMETER = 6
FT_INVALID_BAUD_RATE = 7

FT_DEVICE_NOT_OPENED_FOR_ERASE = 8
FT_DEVICE_NOT_OPENED_FOR_WRITE = 9
FT_FAILED_TO_WRITE_DEVICE = 10
FT_EEPROM_READ_FAILED = 11
FT_EEPROM_WRITE_FAILED = 12
FT_EEPROM_ERASE_FAILED = 13
FT_EEPROM_NOT_PRESENT = 14
FT_EEPROM_NOT_PROGRAMMED = 15
FT_INVALID_ARGS = 16
FT_NOT_SUPPORTED = 17
FT_OTHER_ERROR = 18
FT_DEVICE_LIST_NOT_READY = 19


# /*********************/
def FT_SUCCESS(status):
    result = True if status == FT_OK else False
    return result


# /*********************/


##############################################################
#
# FT_OpenEx Flags
#
FT_OPEN_BY_SERIAL_NUMBER = 1
FT_OPEN_BY_DESCRIPTION = 2
FT_OPEN_BY_LOCATION = 4
FT_OPEN_MASK = FT_OPEN_BY_SERIAL_NUMBER | FT_OPEN_BY_DESCRIPTION | FT_OPEN_BY_LOCATION


##############################################################
#
# Stop Bits
#
FT_STOP_BITS_1 = 0
FT_STOP_BITS_2 = 2

##############################################################
#
# Parity
#
FT_PARITY_NONE = 0
FT_PARITY_ODD = 1
FT_PARITY_EVEN = 2
FT_PARITY_MARK = 3
FT_PARITY_SPACE = 4

##############################################################
#
# Flow Control
#
FT_FLOW_NONE = 0x0000
FT_FLOW_RTS_CTS = 0x0100
FT_FLOW_DTR_DSR = 0x0200
FT_FLOW_XON_XOFF = 0x0400

##############################################################
#
# Purge rx and tx buffers
#
FT_PURGE_RX = 1
FT_PURGE_TX = 2


##############################################################
#
# Timeouts
#
FT_DEFAULT_RX_TIMEOUT = 300
FT_DEFAULT_TX_TIMEOUT = 300

##############################################################
#
# Bit Modes
#
FT_BITMODE_RESET = 0x00
FT_BITMODE_ASYNC_BITBANG = 0x01
FT_BITMODE_MPSSE = 0x02
FT_BITMODE_SYNC_BITBANG = 0x04
FT_BITMODE_MCU_HOST = 0x08
FT_BITMODE_FAST_SERIAL = 0x10
FT_BITMODE_CBUS_BITBANG = 0x20
FT_BITMODE_SYNC_FIFO = 0x40


##############################################################
