#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import struct
import array


##########################################################
##########################################################
##########################################################
def copy_bytearray(source, length, index=0):
    value = bytearray(length)

    for i in range(length):
        value[i] = source[index + i]

    return value


##########################################################
def toInt8(byte_array, index=0):
    work = byte_array[index : index + 1] + b"\0"
    value, dummy = struct.unpack("=bB", work)
    return value


##########################################################
def toIntU8(byte_array, index=0):
    work = byte_array[index : index + 1] + b"\0"
    value, dummy = struct.unpack("=BB", work)
    return value


##########################################################
def toInt16(byte_array, index=0):
    work = byte_array[index : index + 2] + b"\0"
    value, dummy = struct.unpack("=hB", work)
    return value


##########################################################
def toIntU16(byte_array, index=0):
    work = byte_array[index : index + 2] + b"\0"
    value, dummy = struct.unpack("=HB", work)
    return value


##########################################################
def toInt32(byte_array, index=0):
    work = byte_array[index : index + 4] + b"\0"
    value, dummy = struct.unpack("=iB", work)
    return value


##########################################################
def toIntU32(byte_array, index=0):
    work = byte_array[index : index + 4] + b"\0"
    value, dummy = struct.unpack("=IB", work)
    return value


##########################################################
def toInt64(byte_array, index=0):
    work = byte_array[index : index + 8] + b"\0"
    value, dummy = struct.unpack("=qB", work)
    return value


##########################################################
def toIntU64(byte_array, index=0):
    work = byte_array[index : index + 8] + b"\0"
    value, dummy = struct.unpack("=QB", work)
    return value


##########################################################
##########################################################
##########################################################
def getBytes(num, nbytes):
    value = bytearray(nbytes)

    for i in range(nbytes):
        num0 = (num >> (i * 8)) & 0x00FF
        value[i] = num0

    if sys.byteorder == "big":
        value.reverse()

    return value


##########################################################
def getBytesInt8(num):  # signed
    return getBytes(num, 1)


def getBytesIntU8(num):  # unsigned
    return getBytes(num, 1)


##########################################################
def getBytesInt16(num):  # signed
    return getBytes(num, 2)


def getBytesIntU16(num):  # unsigned
    return getBytes(num, 2)


##########################################################
def getBytesInt32(num):  # signed
    return getBytes(num, 4)


def getBytesIntU32(num):  # unsigned
    return getBytes(num, 4)


##########################################################
def getBytesInt48(num):  # signed
    return getBytes(num, 6)


def getBytesIntU48(num):  # unsigned
    return getBytes(num, 6)


##########################################################
def getBytesInt64(num):  # signed
    return getBytes(num, 8)


def getBytesIntU64(num):  # unsigned
    return getBytes(num, 8)


##########################################################
##########################################################
##########################################################
def toSignedNumber(num, nbytes):
    value = 0

    if nbytes > 0:
        msb = 2 ** ((nbytes * 8) - 1)

        if (num & msb) != 0:
            value0 = num & (msb - 1)
            value = (msb * -1) + value0
        else:
            value = num
    else:
        value = None

    return value


##########################################################
def inRangeAndCoarce(value, min_value, max_value):
    if min_value <= value and value <= max_value:
        result = value
    else:
        result = min_value if min_value > value else max_value

    return result


##########################################################
##########################################################
##########################################################
def copyToArray(type0, source):
    dest = array.array(type0)

    nbytes = len(source)
    itemsize = dest.itemsize
    counts0 = range(itemsize)

    for i in range(0, nbytes, itemsize):
        work = 0
        for j in counts0:
            work |= source[i + j] << (j * 8)
        dest.append(work)

    return dest


##########################################################
def copyToArrayU32(source):
    return copyToArray("I", source)
